import os

import numpy
from matplotlib import pyplot

from pygazeanalyser.eyetribereader \
    import read_eyetribe
from pygazeanalyser import gazeplotter


# # # # #
# CONSTANTS

# Get the directory in which this file is.
DIR = os.path.dirname( \
    os.path.abspath(__file__))

# Construct the data directory.
DATADIR = os.path.join(DIR, 'data')

# Throw an Exception when the data
# directory doesn't exist.
if not os.path.isdir(DATADIR):
    raise Exception("ERROR: Could not find data directory!")

# Construct the image directory.
IMGDIR = os.path.join(DIR, 'images')

# Throw an Exception when the image
# directory doesn't exist.
if not os.path.isdir(IMGDIR):
    raise Exception("ERROR: Could not find image directory!")

# Construct the path to the output
# directory.
OUTDIR = os.path.join(DIR, 'output')

# Create the output directory if it
# doesn't exist.
if not os.path.isdir(OUTDIR):
    os.mkdir(OUTDIR)

# A list of this experiment's participants.
PARTICIPANTS = ['edwin']

# The experiment's display resolution.
DISPSIZE = (1920, 1080)

# The string message (event) that was
# used in the experiment to signal the
# start and end of each trial.
START = "IMGONSET"
STOP = "IMGOFFSET"


# # # # #
# ANALYSE

# Loop through all participants.
for i, name in enumerate(PARTICIPANTS):
    
    print("Processing '%s' (%d / %d)" % \
        (name, i+1, len(PARTICIPANTS)))
    
    # Construct the name to the 
    # log files.
    logfile = os.path.join(DATADIR, \
        '%s.txt' % (name))
    eyefile = os.path.join(DATADIR, \
        '%s.tsv' % (name))
    
    # Load the log data.
    raw = numpy.loadtxt(logfile, \
        dtype=str, unpack=True)
    # Put the data in a dict.
    logdata = {}
    for j in range(len(raw)):
        # The first index is the header
        # (with the variable name).
        varname = raw[j][0]
        # The rest of the data is from
        # each trial.
        values = raw[j][1:]
        # Try to convert the values to
        # numbers. If it doesn't work,
        # simply store the values as
        # strings.
        try:
            logdata[varname] = \
                values.astype(float)
        except:
            logdata[varname] = values
    
    # Load the gaze data.
    data = read_eyetribe(eyefile, \
        START, stop=STOP)
    
    # Loop through all trials.
    for j in range(len(logdata)):
        
        # Get the image name.
        imgname = logdata['trial'][j]
        imgpath = os.path.join(IMGDIR, \
            imgname)
        
        # Get the fixations for this
        # trial.
        fix = data[j]['events']['Efix']
        
        # Construct the path to the
        # new plot.
        plotpath = os.path.join( \
            OUTDIR, 'fixations_%s' \
            % imgname)
        # Draw a plot with fixations.
        fig = gazeplotter.draw_fixations( \
            fix, \
            DISPSIZE, \
            imagefile=imgpath, \
            durationsize=True, \
            durationcolour=False, \
            savefilename=plotpath)
        pyplot.close(fig)

        # Construct the path to the
        # new plot.
        plotpath = os.path.join( \
            OUTDIR, 'heatmap_%s' \
            % imgname)
        # Draw a plot with fixations.
        fig = gazeplotter.draw_heatmap( \
            fix, \
            DISPSIZE, \
            imagefile=imgpath, \
            durationweight=True, \
            alpha=0.5, \
            savefilename=plotpath)
        pyplot.close(fig)
